import spidev  # pylint: disable=I0011,F0401
import RPi.GPIO


class AD7843(object):
    """AD7843 class"""
    def __init__(self, width, height, int_pin=None, callback=None, spi=0, speed=2000000):
        self.width = width
        self.height = height
        self.spi = spidev.SpiDev()
        self.spi.open(spi, 0)
        self.spi.max_speed_hz = speed
        self.spi.mode = 0
        self.correction = {
            'x': 364,
            'y': 430,
            'ratio_x': 14.35,
            'ratio_y': 10.59
        }
        self.int_pin = int_pin
        self.callback = callback
        self.bouncetime = 500

    def init(self):
        """some init functions"""
        if self.int_pin:
            RPi.GPIO.setup(self.int_pin, RPi.GPIO.IN)
            RPi.GPIO.add_event_detect(
                self.int_pin, RPi.GPIO.BOTH, callback=self._interrupt, bouncetime=self.bouncetime
            )

    def get_x(self, value):
        """correct value to x"""
        return self.width - int((value - self.correction['x']) / self.correction['ratio_x'])

    def get_y(self, value):
        """correct value to y"""
        return self.height - int((value - self.correction['y']) / self.correction['ratio_y'])

    def _interrupt(self, channel):
        """call users callback"""
        self.callback(self.get_position())

    def get_position(self):
        """get touch coords"""
        buffer = []
        while len(buffer) < 20:
            self.spi.xfer2([0xd0])
            recvx = self.spi.readbytes(2)
            self.spi.xfer2([0x90])
            recvy = self.spi.readbytes(2)

            tc_rx = recvx[0] << 5
            tc_rx |= recvx[1] >> 3

            tc_ry = recvy[0] << 5
            tc_ry |= recvy[1] >> 3

            pos_x = self.get_x(tc_rx)
            pos_y = self.get_y(tc_ry)
            if 0 <= pos_x <= self.width and 0 <= pos_y <= self.height:
                buffer.append((pos_x, pos_y))

        return self._calculate_avr(buffer)

    def _calculate_avr(self, points):
        """calculate x,y by average"""
        sum_x = 0
        sum_y = 0
        for point in points:
            sum_x += point[0]
            sum_y += point[1]

        return int(sum_x / len(points)), int(sum_y / len(points))

    def close(self):
        """close action"""
        if self.int_pin:
            RPi.GPIO.remove_event_detect(self.int_pin)
        self.spi.close()




"""NJU6450 chip"""
from gfxlcd.drawing.page import Page
from gfxlcd.abstract.chip import Chip


class NJU6450(Page, Chip):
    """Class for an LCD with NJU6450 chip"""
    def __init__(self, width, height, driver, auto_flush=True):
        Chip.__init__(self, width, height, driver, auto_flush)
        Page.__init__(self, driver)

    def init(self):
        """initialize display"""
        self.driver.init()
        Page.init(self)
        Chip.init(self)
        self.driver.reset()

        init_sequence = [0xae, 0xa4, 0xa9, 0xe2, 0xa0, 0xaf]
        for cmd in init_sequence:
            self.driver.cmd(cmd, 0)
            self.driver.cmd(cmd, 1)

    def set_xy(self, pos_x, pos_y):
        """set xy pos"""
        if pos_x < self.width/2:
            self.driver.cmd(0xB8 | pos_y, 0)
            self.driver.cmd(0x00 | pos_x, 0)
        else:
            self.driver.cmd(0xB8 | pos_y, 1)
            self.driver.cmd(0x00 | (pos_x - self.width//2), 1)

    def _converted_background_color(self):
        """convert RGB background to available color"""
        return 1

    def _converted_color(self):
        """convert RGB color to available color"""
        return 1

    def flush(self, force=None):
        """flush buffer to device
        :force - boolean|None"""
        if force is None:
            force = self.options['auto_flush']

        if force:
            for j in range(0, self.height//8):
                for i in range(0, self.width):
                    self.set_xy(i, j)
                    if i < self.width/2:
                        self.driver.data(self.get_page_value(i, j), 0)
                    else:
                        self.driver.data(self.get_page_value(i, j), 1)



"""NJU6450 gpio driver class"""
import time
import RPi.GPIO  # pylint: disable=I0011,F0401
from gfxlcd.abstract.driver import Driver

RPi.GPIO.setmode(RPi.GPIO.BCM)


class GPIO(Driver):
    """GPIO driver"""
    def __init__(self):
        self.pins = {
            'A0': 17,
            'E1': 22,
            'E2': 21,
            'D0': 23,
            'D1': 24,
            'D2': 25,
            'D3': 12,
            'D4': 16,
            'D5': 20,
            'D6': 26,
            'D7': 19,
            'RST': 5,
        }

        self.data_pins = [
            'D0', 'D1', 'D2', 'D3', 'D4', 'D5', 'D6', 'D7'
        ]

    def init(self):
        """init a device"""
        for pin in self.pins:
            RPi.GPIO.setup(self.pins[pin], RPi.GPIO.OUT)
            RPi.GPIO.output(self.pins[pin], 0)

    def reset(self):
        """resets a device"""
        RPi.GPIO.output(self.pins['RST'], 1)
        time.sleep(0.025)
        RPi.GPIO.output(self.pins['RST'], 0)
        time.sleep(0.025)
        RPi.GPIO.output(self.pins['RST'], 1)
        time.sleep(0.025)
        RPi.GPIO.output(self.pins['A0'], 0)
        RPi.GPIO.output(self.pins['E1'], 1)
        RPi.GPIO.output(self.pins['E2'], 1)

    def cmd(self, data, enable):
        """send command"""
        RPi.GPIO.output(self.pins['A0'], 0)
        self.send(data, enable)

    def data(self, data, enable):
        """send data"""
        RPi.GPIO.output(self.pins['A0'], 1)
        self.send(data, enable)

    def send(self, data, enable):
        """Write to gpio"""
        RPi.GPIO.output(self.pins['E1'], 0)
        RPi.GPIO.output(self.pins['E2'], 0)
        for i in self.data_pins:
            value = data & 0x01
            RPi.GPIO.output(self.pins[i], value)
            data >>= 1

        RPi.GPIO.output(self.pins['E'+str(enable+1)], 1)
        time.sleep(0.00025)
        RPi.GPIO.output(self.pins['E1'], 0)
        RPi.GPIO.output(self.pins['E2'], 0)



# -*- coding: utf-8 -*-

"""setup for GfxLCD package"""

import os
from setuptools import setup, find_packages


def read(*paths):
    """Build a file path from *paths* and return the contents."""
    with open(os.path.join(*paths), 'r') as file_handler:
        return file_handler.read()

setup(
    name='gfxlcd',
    version='0.2.1',
    description='gfxlcd is a handler for graphical lcds: ILI9328, SSD1306, NJU6450, touch panel: AD7843 @ Raspberry Pi.',
    keywords=['gfxlcd', 'raspberry pi' ,'ili9328' ,'ssd1306', 'nju6450', 'lcd', 'graphical lcd', 'touch panel', 'ad7843'],
    long_description=(read('readme.md')),
    url='https://github.com/bkosciow/gfxlcd',
    license='MIT',
    author='Bartosz Kościów',
    author_email='kosci1@gmail.com',
    py_modules=['gfxlcd'],
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Home Automation'
    ],
    packages=find_packages(exclude=['tests*']),
)



"""ILI9325 chip driver"""
import time
from gfxlcd.drawing.area import Area
from gfxlcd.abstract.chip import Chip


class ILI9325(Area, Chip):
    """Class for ILI9325 based LCD"""
    def __init__(self, width, height, driver):
        Chip.__init__(self, width, height, driver, True)
        Area.__init__(self, driver)

    def _converted_background_color(self):
        """color from 8-8-8 to 5-6-5"""
        rgb = self.options['background_color']['R'] << 16 | \
            self.options['background_color']['G'] << 8 | \
            self.options['background_color']['B']
        return ((rgb & 0x00f80000) >> 8) |\
            ((rgb & 0x0000fc00) >> 5) | ((rgb & 0x000000f8) >> 3)

    def _converted_color(self):
        """color from 8-8-8 to 5-6-5"""
        rgb = self.options['color']['R'] << 16 | \
            self.options['color']['G'] << 8 | \
            self.options['color']['B']
        return ((rgb & 0x00f80000) >> 8) |\
            ((rgb & 0x0000fc00) >> 5) | ((rgb & 0x000000f8) >> 3)

    def init(self):
        """init display"""
        self.driver.init()
        Area.init(self)
        Chip.init(self)
        self.driver.reset()

        # ************* ILI9325C/D **********
        # set SS and SM bit
        self.driver.cmd(0x0001, None)
        self.driver.data(0x0100, None)
        # set 1 line inversion
        self.driver.cmd(0x0002, None)
        self.driver.data(0x0200, None)
        # set GRAM write direction and BGR=1
        self.driver.cmd(0x0003, None)
        self.driver.data(0x1030, None)
        # Resize register
        self.driver.cmd(0x0004, None)
        self.driver.data(0x0000, None)
        # set the back porch and front porch
        self.driver.cmd(0x0008, None)
        self.driver.data(0x0207, None)
        # set non-display area refresh cycle ISC[3:0]
        self.driver.cmd(0x0009, None)
        self.driver.data(0x0000, None)
        # FMARK function
        self.driver.cmd(0x000A, None)
        self.driver.data(0x0000, None)
        # RGB interface setting
        self.driver.cmd(0x000C, None)
        self.driver.data(0x0000, None)
        # Frame marker Position
        self.driver.cmd(0x000D, None)
        self.driver.data(0x0000, None)
        # RGB interface polarity
        self.driver.cmd(0x000F, None)
        self.driver.data(0x0000, None)

        # ************* Power On sequence ****************
        # SAP, BT[3:0], AP, DSTB, SLP, STB
        self.driver.cmd(0x0010, None)
        self.driver.data(0x0000, None)
        # DC1[2:0], DC0[2:0], VC[2:0]
        self.driver.cmd(0x0011, None)
        self.driver.data(0x0007, None)
        # VREG1OUT voltage
        self.driver.cmd(0x0012, None)
        self.driver.data(0x0000, None)
        # VDV[4:0] for VCOM amplitude
        self.driver.cmd(0x0013, None)
        self.driver.data(0x0000, None)
        self.driver.cmd(0x0007, None)
        self.driver.data(0x0001, None)
        time.sleep(0.2)  # Dis-charge capacitor power voltage
        # SAP, BT[3:0], AP, DSTB, SLP, STB
        self.driver.cmd(0x0010, None)
        self.driver.data(0x1690, None)
        # Set DC1[2:0], DC0[2:0], VC[2:0]
        self.driver.cmd(0x0011, None)
        self.driver.data(0x0227, None)
        time.sleep(0.05)
        self.driver.cmd(0x0012, None)
        self.driver.data(0x000D, None)
        time.sleep(0.05)
        # VDV[4:0] for VCOM amplitude
        self.driver.cmd(0x0013, None)
        self.driver.data(0x1200, None)
        # 04  VCM[5:0] for VCOMH
        self.driver.cmd(0x0029, None)
        self.driver.data(0x000A, None)
        # Set Frame Rate
        self.driver.cmd(0x002B, None)
        self.driver.data(0x000D, None)
        time.sleep(0.05)
        # GRAM horizontal Address
        self.driver.cmd(0x0020, None)
        self.driver.data(0x0000, None)
        # GRAM Vertical Address
        self.driver.cmd(0x0021, None)
        self.driver.data(0x0000, None)

        # ************* Adjust the Gamma Curve *************
        self.driver.cmd(0x0030, None)
        self.driver.data(0x0000, None)
        self.driver.cmd(0x0031, None)
        self.driver.data(0x0404, None)
        self.driver.cmd(0x0032, None)
        self.driver.data(0x0003, None)
        self.driver.cmd(0x0035, None)
        self.driver.data(0x0405, None)
        self.driver.cmd(0x0036, None)
        self.driver.data(0x0808, None)
        self.driver.cmd(0x0037, None)
        self.driver.data(0x0407, None)
        self.driver.cmd(0x0038, None)
        self.driver.data(0x0303, None)
        self.driver.cmd(0x0039, None)
        self.driver.data(0x0707, None)
        self.driver.cmd(0x003C, None)
        self.driver.data(0x0504, None)
        self.driver.cmd(0x003D, None)
        self.driver.data(0x0808, None)

        # ************* Set GRAM area *************
        # Horizontal GRAM Start Address
        self.driver.cmd(0x0050, None)
        self.driver.data(0x0000, None)
        # Horizontal GRAM End Address
        self.driver.cmd(0x0051, None)
        self.driver.data(0x00EF, None)
        # Vertical GRAM Start Address
        self.driver.cmd(0x0052, None)
        self.driver.data(0x0000, None)
        # Vertical GRAM Start Address
        self.driver.cmd(0x0053, None)
        self.driver.data(0x013F, None)
        # Gate Scan Line
        self.driver.cmd(0x0060, None)
        self.driver.data(0xA700, None)
        # NDL, VLE, REV
        self.driver.cmd(0x0061, None)
        self.driver.data(0x0001, None)
        # set scrolling line
        self.driver.cmd(0x006A, None)
        self.driver.data(0x0000, None)

        # ************* Partial Display Control *************
        self.driver.cmd(0x0080, None)
        self.driver.data(0x0000, None)
        self.driver.cmd(0x0081, None)
        self.driver.data(0x0000, None)
        self.driver.cmd(0x0082, None)
        self.driver.data(0x0000, None)
        self.driver.cmd(0x0083, None)
        self.driver.data(0x0000, None)
        self.driver.cmd(0x0084, None)
        self.driver.data(0x0000, None)
        self.driver.cmd(0x0085, None)
        self.driver.data(0x0000, None)

        # ************* Panel Control *************
        self.driver.cmd(0x0090, None)
        self.driver.data(0x0010, None)
        self.driver.cmd(0x0092, None)
        self.driver.data(0x0000, None)
        # 262K color and display ON
        self.driver.cmd(0x0007, None)
        self.driver.data(0x0133, None)
